@@ -1,16 +1,15 @@
-#!/bin/sh
-# the next line restarts using wish \
-exec wish "$0" ${1+"$@"}
+#!/usr/bin/env wish
 
 #==============================================================================
 # Demonstrates the interactive tablelist cell editing with the aid of some
-# widgets from the BWidget package and of the Tk core checkbutton widget.
+# widgets from the BWidget package and of the Tk core checkbutton and
+# menubutton widgets.
 #
-# Copyright (c) 2004-2008  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2004-2021  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
-package require Tk 8.3				;# because of entry validation
-package require Tablelist
+package require Tk 8.4				;# because of "-compound"
+package require tablelist 6.15
 package require BWidget
 
 wm title . "Serial Line Configuration"
@@ -20,9 +19,13 @@ wm title . "Serial Line Configuration"
 #
 set dir [file dirname [info script]]
 source [file join $dir option.tcl]
-option add *Tablelist*Checkbutton.background		white
-option add *Tablelist*Checkbutton.activeBackground	white
-option add *Tablelist*Entry.background			white
+option add *Tablelist*Entry.background white
+
+#
+# Create the images "checkedImg" and "uncheckedImg", as well as 16 images of
+# names like "img#FF0000", displaying colors identified by names like "red"
+#
+source [file join $dir images.tcl]
 
 #
 # Register some widgets from the BWidget package for interactive cell editing
@@ -31,12 +34,6 @@ tablelist::addBWidgetEntry
 tablelist::addBWidgetSpinBox
 tablelist::addBWidgetComboBox
 
-#
-# Create two images, to be displayed in tablelist cells with boolean values
-#
-set checkedImg   [image create photo -file [file join $dir checked.gif]]
-set uncheckedImg [image create photo -file [file join $dir unchecked.gif]]
-
 #
 # Create a tablelist widget with editable columns (except the first one)
 #
@@ -51,7 +48,8 @@ tablelist::tablelist $tbl \
 	      0 "Stop Bits"	  center
 	      0 "Handshake"	  left
 	      0 "Activation Date" center
-	      0 "Activation Time" center} \
+	      0 "Activation Time" center
+	      0 "Cable Color"	  center} \
     -editstartcommand editStartCmd -editendcommand editEndCmd \
     -height 0 -width 0
 if {[$tbl cget -selectborderwidth] == 0} {
@@ -59,7 +57,7 @@ if {[$tbl cget -selectborderwidth] == 0} {
 }
 $tbl columnconfigure 0 -sortmode integer
 $tbl columnconfigure 1 -name available -editable yes -editwindow checkbutton \
-    -formatcommand emptyStr
+    -formatcommand emptyStr -labelwindow checkbutton
 $tbl columnconfigure 2 -name lineName  -editable yes -editwindow Entry \
     -sortmode dictionary
 $tbl columnconfigure 3 -name baudRate  -editable yes -editwindow ComboBox \
@@ -72,32 +70,25 @@ $tbl columnconfigure 8 -name actDate   -editable yes -editwindow Entry \
     -formatcommand formatDate -sortmode integer
 $tbl columnconfigure 9 -name actTime   -editable yes -editwindow Entry \
     -formatcommand formatTime -sortmode integer
+$tbl columnconfigure 10 -name color    -editable yes -editwindow menubutton \
+    -formatcommand emptyStr
 
 proc emptyStr   val { return "" }
 proc formatDate val { return [clock format $val -format "%Y-%m-%d"] }
 proc formatTime val { return [clock format $val -format "%H:%M:%S"] }
 
 #
-# Populate the tablelist widget; set the activation
-# date & time to 10 minutes past the current clock value
+# Populate the tablelist widget and configure the checkbutton
+# embedded into the header label of the column "available"
 #
-set clock [clock seconds]
-incr clock 600
-for {set n 1} {$n <= 8} {incr n} {
-    $tbl insert end [list $n 1 "Line $n" 9600 8 None 1 XON/XOFF $clock $clock]
-    $tbl cellconfigure end,available -image $checkedImg
-}
-for {set n 9} {$n <= 16} {incr n} {
-    $tbl insert end [list $n 0 "Line $n" 9600 8 None 1 XON/XOFF $clock $clock]
-    $tbl cellconfigure end,available -image $uncheckedImg
-}
+source [file join $dir serialParams.tcl]
 
 set btn [button .btn -text "Close" -command exit]
 
 #
 # Manage the widgets
 #
-pack $btn -side bottom -pady 10
+pack $btn -side bottom -pady 7p
 pack $tbl -side top -expand yes -fill both
 
 #------------------------------------------------------------------------------
@@ -174,6 +165,20 @@ proc editStartCmd {tbl row col text} {
 	    $w configure -invalidcommand bell -validate key -validatecommand \
 		{expr {[string length %P] <= 8 && [regexp {^[0-9:]*$} %S]}}
 	}
+
+	color {
+	    #
+	    # Populate the menu and make sure the menubutton will display the
+	    # color name rather than $text, which is "", due to -formatcommand
+	    #
+	    set menu [$w cget -menu]
+	    foreach name $::colorNames {
+		$menu add radiobutton -compound left \
+		    -image img$::colors($name) -label $name
+	    }
+	    $menu entryconfigure 8 -columnbreak 1
+	    return [$tbl cellcget $row,$col -text]
+	}
     }
 
     return $text
@@ -183,7 +188,7 @@ proc editStartCmd {tbl row col text} {
 # editEndCmd
 #
 # Performs a final validation of the text contained in the edit window and gets
-# the cell's internal contents.
+# the cell's internal content.
 #------------------------------------------------------------------------------
 proc editEndCmd {tbl row col text} {
     switch [$tbl columncget $col -name] {
@@ -191,7 +196,7 @@ proc editEndCmd {tbl row col text} {
 	    #
 	    # Update the image contained in the cell
 	    #
-	    set img [expr {$text ? $::checkedImg : $::uncheckedImg}]
+	    set img [expr {$text ? "checkedImg" : "uncheckedImg"}]
 	    $tbl cellconfigure $row,$col -image $img
 	}
 
@@ -201,7 +206,7 @@ proc editEndCmd {tbl row col text} {
 	    #
 	    if {![regexp {^[0-9]+$} $text] || $text < 50 || $text > 921600} {
 		bell
-		tk_messageBox -title Error -icon error -type ok -message \
+		tk_messageBox -title "Error" -icon error -message \
 		    "The baud rate must be an integer in the range 50..921600"
 		$tbl rejectinput
 	    }
@@ -213,8 +218,7 @@ proc editEndCmd {tbl row col text} {
 	    #
 	    if {[catch {clock scan $text} actDate] != 0} {
 		bell
-		tk_messageBox -title Error -icon error -type ok -message \
-		    "Invalid date"
+		tk_messageBox -title "Error" -icon error -message "Invalid date"
 		$tbl rejectinput
 		return ""
 	    }
@@ -228,7 +232,7 @@ proc editEndCmd {tbl row col text} {
 	    set actClock [clock scan [formatTime $actTime] -base $actDate]
 	    if {$actClock <= [clock seconds]} {
 		bell
-		tk_messageBox -title Error -icon error -type ok -message \
+		tk_messageBox -title "Error" -icon error -message \
 		    "The activation date & time must be in the future"
 		$tbl rejectinput
 	    } else {
@@ -244,8 +248,7 @@ proc editEndCmd {tbl row col text} {
 	    set actDate [$tbl cellcget $row,actDate -text]
 	    if {[catch {clock scan $text -base $actDate} actClock] != 0} {
 		bell
-		tk_messageBox -title Error -icon error -type ok -message \
-		    "Invalid time"
+		tk_messageBox -title "Error" -icon error -message "Invalid time"
 		$tbl rejectinput
 		return ""
 	    }
@@ -257,7 +260,7 @@ proc editEndCmd {tbl row col text} {
 	    #
 	    if {$actClock <= [clock seconds]} {
 		bell
-		tk_messageBox -title Error -icon error -type ok -message \
+		tk_messageBox -title "Error" -icon error -message \
 		    "The activation date & time must be in the future"
 		$tbl rejectinput
 	    } else {
@@ -265,6 +268,13 @@ proc editEndCmd {tbl row col text} {
 		return $actClock
 	    }
 	}
+
+	color {
+	    #
+	    # Update the image contained in the cell
+	    #
+	    $tbl cellconfigure $row,$col -image img$::colors($text)
+	}
     }
 
     return $text
