@@ -1,15 +1,14 @@
 #==============================================================================
 # Main Tablelist package module.
 #
-# Copyright (c) 2000-2008  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
+# Copyright (c) 2000-2021  Csaba Nemethi (E-mail: csaba.nemethi@t-online.de)
 #==============================================================================
 
-package require Tcl 8
-package require Tk  8
-package require -exact tablelist::common 4.10
+package require Tk 8
+package require -exact tablelist::common 6.15.1
 
-package provide Tablelist $::tablelist::version
 package provide tablelist $::tablelist::version
+package provide Tablelist $::tablelist::version
 
 ::tablelist::useTile 0
 ::tablelist::createBindings
